package callbacks;

import java.rmi.AlreadyBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import callbacks.Client.ClientInterface;

public class Server {

	static interface ServerInterface extends Remote {
		void sayHelloOnServerRemote(ClientInterface client) throws RemoteException;
	}

	static class ServerImpl extends UnicastRemoteObject implements ServerInterface {

		private static final long serialVersionUID = 1L;

		protected ServerImpl(int port) throws RemoteException {
			super(port);
		}

		@Override
		public void sayHelloOnServerRemote(ClientInterface client) throws RemoteException {
			System.out.println("Hello from Server");
			// callback to client, this will create a proxy on the server and call the
			// sekeleton on the client
			client.sayHelloOnClientRemote();
		}

	}

	public static void main(String[] args) {
		try {
			ServerImpl remoteObject = new ServerImpl(49152);
			Registry registry = LocateRegistry.createRegistry(49152);
			registry.bind("ServerObject", remoteObject);
			System.out.println("Server Ready...");
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (AlreadyBoundException e) {
			e.printStackTrace();
		}
	}
}
